﻿using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Windows;
using BingExplorer.Services;
using Microsoft.Phone.Scheduler;
using Microsoft.Phone.Shell;

namespace BingExplorer.Task
{
    public class ScheduledAgent : ScheduledTaskAgent
    {
        private static volatile bool _classInitialized;

        /// <remarks>
        /// ScheduledAgent constructor, initializes the UnhandledException handler
        /// </remarks>
        public ScheduledAgent()
        {
            if (!_classInitialized)
            {
                _classInitialized = true;
                // Subscribe to the managed exception handler
                Deployment.Current.Dispatcher.BeginInvoke(delegate
                {
                    Application.Current.UnhandledException += ScheduledAgent_UnhandledException;
                });
            }
        }

        /// Code to execute on Unhandled Exceptions
        private void ScheduledAgent_UnhandledException(object sender, ApplicationUnhandledExceptionEventArgs e)
        {
            if (System.Diagnostics.Debugger.IsAttached)
            {
                // An unhandled exception has occurred; break into the debugger
                System.Diagnostics.Debugger.Break();
            }
        }

        /// <summary>
        /// Agent that runs a scheduled task
        /// </summary>
        /// <param name="task">
        /// The invoked task
        /// </param>
        /// <remarks>
        /// This method is called when a periodic or resource intensive task is invoked
        /// </remarks>
        protected override void OnInvoke(ScheduledTask task)
        {
            // Ottengo la lista delle immagini
            BingImagesClient client = new BingImagesClient();
            client.LoadCompleted += client_LoadCompleted;
            client.LoadAsync();
        }

        private void client_LoadCompleted(object sender, LoadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                NotifyComplete();
                return;
            }

            // Scelgo randomicamente la foto
            int i = new Random(Environment.TickCount).Next(0, e.Images.Count() - 1);
            Uri backgroundImage = new Uri(e.Images.ElementAt(i).Thumbnail);

            // Scarico l'immagine
            WebClient client = new WebClient();
            client.OpenReadCompleted += Image_OpenReadCompleted;
            client.OpenReadAsync(backgroundImage);
        }

        private void Image_OpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                NotifyComplete();
                return;
            }

            const string fileName = "Shared/ShellContent/task.jpg";

            // Travaso lo stream dell'immagine nell'isolated storage
            using (IsolatedStorageFile storage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (Stream tileStream = storage.CreateFile(fileName))
                {
                    e.Result.CopyTo(tileStream);
                }
            }

            // Aggiorno la tile
            StandardTileData data = new StandardTileData
                                        {
                                            BackgroundImage = new Uri("isostore:/" + fileName),
                                        };

            ShellTile firstTile = ShellTile.ActiveTiles.First();
            firstTile.Update(data);

#if DEBUG
            ScheduledActionService.LaunchForTest("refresh", TimeSpan.FromSeconds(60));
#endif

            NotifyComplete();
        }
    }
}