﻿using System;
using System.Collections.Generic;
using System.IO;
using System.IO.IsolatedStorage;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using BingExplorer.Services;
using Microsoft.Phone;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Shell;
using Microsoft.Phone.Tasks;
using Microsoft.Xna.Framework.Media;
using System.Windows.Media.Imaging;

namespace BingExplorer
{
    public partial class DetailPage : PhoneApplicationPage
    {

        public DetailPage()
        {
            InitializeComponent();
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            image.UriSource = new Uri(this.NavigationContext.QueryString["u"]);
        }

        private void Share_Click(object sender, EventArgs e)
        {
            ShareLinkTask shareLinkTask = new ShareLinkTask();
            shareLinkTask.Title = "Immagine bing";
            shareLinkTask.LinkUri = image.UriSource;
            shareLinkTask.Show();
        }

        private void Pin_Click(object sender, EventArgs e)
        {
            // Creo l'uri verso l'immagine (ridimensionata)
            Uri backgroundImage = new Uri(BingImage.GetThumbnailUri(image.UriSource.ToString()));

            WebClient client = new WebClient();
            client.OpenReadCompleted += Image_OpenReadCompleted;
            client.OpenReadAsync(backgroundImage);           
        }

        private void Image_OpenReadCompleted(object sender, OpenReadCompletedEventArgs e)
        {
            if (e.Error != null)
            {
                MessageBox.Show("Errore nel download dell'immagine");
                return;
            }

            string fileName = "Shared/ShellContent/" + Guid.NewGuid().ToString("N") + ".jpg";

            // Travaso lo stream dell'immagine nell'isolated storaged
            using (IsolatedStorageFile storage = IsolatedStorageFile.GetUserStoreForApplication())
            {
                using (Stream tileStream = storage.CreateFile(fileName))
                {
                    e.Result.CopyTo(tileStream);
                }
            }

            ShellTile tile = ShellTile.ActiveTiles.FirstOrDefault(t => t.NavigationUri == this.NavigationService.Source);
            // Cancello l'eventuale tile esistente
            if (tile != null)
                tile.Delete();

            Uri backgroundImage = new Uri("isostore:/" + fileName);

            // Configurazione della tile
            ShellTileData data = new StandardTileData
            {
                BackgroundImage = backgroundImage,
                BackBackgroundImage = backgroundImage,
                Title = "BingExplorer",
                BackContent = DateTime.Now.ToShortDateString()
            };
            ShellTile.Create(this.NavigationService.Source, data);
        }

        private void Save_Click(object sender, EventArgs e)
        {
            using (Stream imageStream = new MemoryStream())
            {
                // Carico l'immagine in pixell
                WriteableBitmap bitmap = new WriteableBitmap(image);
                // Salvo in jpeg
                bitmap.SaveJpeg(imageStream, bitmap.PixelWidth, bitmap.PixelHeight, 0, 95);

                // Azzero lo stream
                imageStream.Position = 0;

                // Salvo l'immagine nella libreria
                using (MediaLibrary library = new MediaLibrary())
                {
                    string name = System.IO.Path.GetFileNameWithoutExtension(this.image.UriSource.AbsolutePath);
                    library.SavePicture(name, imageStream);
                }

                MessageBox.Show("Immagine salvata nella libreria immagini");
            }

        }
    }
}