﻿using System;
using System.Linq;
using System.Runtime.Serialization;
using System.Windows;
using System.Windows.Controls;
using BingExplorer.Services;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Net.NetworkInformation;
using Microsoft.Phone.Scheduler;
using Microsoft.Phone.UserData;

namespace BingExplorer
{
    public partial class MainPage : PhoneApplicationPage
    {
        private BingImagesClient bingImagesClient;
        private bool loaded;

        // Constructor
        public MainPage()
        {
            InitializeComponent();

            

            bingImagesClient = new BingImagesClient();
            bingImagesClient.LoadCompleted += new EventHandler<LoadCompletedEventArgs>(bingImagesClient_LoadCompleted);

            InitSchedule();
        }

        private void InitSchedule()
        {
            const string taskName = "refresh";

            if (ScheduledActionService.Find(taskName) != null)
                ScheduledActionService.Remove(taskName);

            PeriodicTask task = new PeriodicTask(taskName);
            task.Description = "Aggiorna le immagini sulla tile";

            try
            {
                // Schedulo il task
                ScheduledActionService.Add(task);

#if DEBUG
                ScheduledActionService.LaunchForTest(taskName, TimeSpan.FromSeconds(5));
#endif
            }
            catch (InvalidOperationException exception)
            {
                // Controllo che il background task non sia disattivato
                if (exception.Message.Contains("BNS Error: The action is disabled"))
                {
                    MessageBox.Show("Impossibile aggiornare le immagini perché le attività in background sono disabilitate per questa applicazione");
                }
            }
        }

        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            base.OnNavigatedTo(e);

            // Carico solo se non l'ho già fatto
            bool instancePreserved = ((App) App.Current).InstancePreserved;

            // Ricarico dallo stato
            if (!instancePreserved && this.State.ContainsKey("data"))
            {
                this.pivot.SelectedIndex = (int) this.State["order"];
                this.DataContext = this.State["data"];

                this.loaded = true;
            }

            if (!this.loaded)
                LoadImages();
        }

        protected override void OnNavigatingFrom(System.Windows.Navigation.NavigatingCancelEventArgs e)
        {
            base.OnNavigatingFrom(e);

            this.State["order"] = pivot.SelectedIndex;
            this.State["data"] = this.DataContext;
        }

        private void LoadImages()
        {
            if (DeviceNetworkInformation.IsNetworkAvailable)
            {
                this.progressIndicator.IsVisible = true;

                bingImagesClient.LoadAsync();
            }
            else
            {
                MessageBox.Show("Internet non disponibile");
            }
        }

        private void bingImagesClient_LoadCompleted(object sender, LoadCompletedEventArgs e)
        {
            // Se è un errore mostro il messaggio
            if (e.Error != null)
            {
                MessageBox.Show(e.Error.Message);
            }
            else
            {
                this.loaded = true;

                // Preparo le liste ordinate e le imposto come contesto dati
                this.DataContext = new ImagesList
                                       {
                                           ByDate = e.Images.OrderByDescending(i => i.Date).ToArray(),
                                           ByTitle = e.Images.OrderBy(i => i.Title).ToArray()
                                       };
            }
            this.progressIndicator.IsVisible = false;
        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            this.LoadImages();
        }

        private void ListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            BingImage bi = e.AddedItems.OfType<BingImage>().FirstOrDefault();
            if (bi != null)
            {
                // Dettaglio immagine
                string u = "/DetailPage.xaml?u=" + Uri.EscapeDataString(bi.Link);
                this.NavigationService.Navigate(new Uri(u, UriKind.Relative));

                // Annullo la selezione
                ((ListBox) sender).SelectedIndex = -1;
            }
        }
    }

    [DataContract]
    public class ImagesList
    {
        [DataMember]
        public BingImage[] ByDate { get; set; }

        [DataMember]
        public BingImage[] ByTitle { get; set; }
    }
}